package com.example.gps_location;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Properties;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.location.Location;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.IBinder;
import android.os.PowerManager;
import android.util.Log;
import android.widget.Toast;


public class GPSTracker extends Service{

	public static final String MY_SERVICE = "com.example.gps_location.GPSTracker"; //jedinecny nazov pre moj service
	private static final String TAG = "TESTGPS";  //pouzivane pri debuggovani
	private LocationManager mLocationManager = null;
	private static final int LOCATION_INTERVAL = 2000;  //minimalny interval medzi zmenou lokacie 2 sekundy
	private static final float LOCATION_DISTANCE = 5;  //minimalny interval medzi zmenou lokacie 5 metrov
	private ArrayList<String> db;                      //arraylist docasnych bodov lokacie pred zapisanim do suboru
	public static String filename;					//jedinecny nazov interneho suboru s datami
	public static final int APPEND=2;				//konstanta pre prilozenie celeho suboru
	public static final int RESUME=1;				//konstanta pre pokracovanie preruseneho prikladania suboru
	private final Object lock_file = new Object();  //lock na subor
	private final Object lock_db = new Object();  	//lock na arraylist
	public static boolean interupted_sending=false;  //flag pre uspesne/neuspesne posielanie dat na server 
	private PowerManager.WakeLock wakeLock=null;
	public static boolean send_because_gps=false;    //flag pre posielanie suboru na server po vypnuti GPS providera
	private BroadcastReceiver networkStateReceiver = new BroadcastReceiver() {

       // @Override
        public void onReceive(Context context, Intent intent) {
        	
        	ConnectivityManager connectivityManager=(ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        	NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
    		//ak sme pripojeny na internet pokusime sa poslat subor s datami na server
        	if (activeNetworkInfo != null)
    			try {
					storeData();
				} catch (IOException e) {
					e.printStackTrace();
				} 
        }
    };
    
    private class LocationListener implements android.location.LocationListener{
        Location mLastLocation;
        public LocationListener(String provider)
        {
            Log.e(TAG, "LocationListener " + provider);
            mLastLocation = new Location(provider);
        }
        public void onLocationChanged(Location location)
        {
            Log.e(TAG, "onLocationChanged: " + location);
            //vzdy pri zmene lokacie si zapiseme novy bod do arraylistu db
            mLastLocation.set(location);
            String text=""+(mLastLocation.getTime())/1000+"|"+mLastLocation.getLatitude()+'|'+mLastLocation.getLongitude()+";";
            synchronized(lock_db){
            	db.add(text);
            	//ak ma arraylist viac ako 10 bodov, zapiseme body do suboru
            	if (db.size()>10)
					try {
						storeData();
					} catch (IOException e) {
						e.printStackTrace();
					}
            }
        }
        public void onProviderDisabled(String provider)
        {
        	//pri vypnuti GPS providera nastavime flag na posielanie suboru send_because_gps
        	send_because_gps=true;
        	releaseWakeLock();  //uvolnime wakelock na procesor
        	//zapiseme si do arraylistu "RESET" (koniec cesty) a body arraylistu zapiseme do suboru
        	synchronized(lock_db){
            	db.add("RESET;");
            	try {
					storeData();
				} catch (IOException e) {
					e.printStackTrace();
				}
            }
            Log.e(TAG, "onProviderDisabled: " + provider); 
        }
        public void onProviderEnabled(String provider)
        {	//pri zapnuti GPS providera ziskame wakelock pre udrzanie procesora v bdelom stave
        	if (mLocationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)){ 
        		acquireWakeLock();
        	}
            Log.e(TAG, "onProviderEnabled: " + provider);
        }
        public void onStatusChanged(String provider, int status, Bundle extras)
        {
        	Log.e(TAG, "onStatusChanged: " + provider);
        }
    } 
    
    LocationListener mLocationListener=  new LocationListener(LocationManager.GPS_PROVIDER);
    
    @Override
    public IBinder onBind(Intent arg0)
    {
        return null;
    }
    @Override
    public int onStartCommand(Intent intent, int flags, int startId)
    {
        Log.e(TAG, "onStartCommand");
        try {
            //inicializujeme LocaionManager a podla stavu GPS providera zobrazime hlasku "zbierame data"/"zapnite si prosim GPS"
        	mLocationManager.requestLocationUpdates(
                    LocationManager.GPS_PROVIDER, LOCATION_INTERVAL, LOCATION_DISTANCE,
                    mLocationListener);
            if (mLocationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)){
            	acquireWakeLock();
            	Toast.makeText(GPSTracker.this, "GPS Location is being gathered" , Toast.LENGTH_SHORT).show();
            } else Toast.makeText(GPSTracker.this, "Please turn on GPS :)" , Toast.LENGTH_SHORT).show();
        } catch (java.lang.SecurityException ex) {
            Log.i(TAG, "fail to request location update, ignore", ex);
            Toast.makeText(GPSTracker.this, "There was some problem please restart application and start again" , Toast.LENGTH_SHORT).show();
        } catch (IllegalArgumentException ex) {
            Log.d(TAG, "gps provider does not exist " + ex.getMessage());
            Toast.makeText(GPSTracker.this, "Please turn on GPS :)" , Toast.LENGTH_SHORT).show();
        }
        super.onStartCommand(intent, flags, startId);       
        return START_STICKY;  //dolezite nastavenie nonstop beh servicu (service sa bude vzdy zapinat, kym sa ho nerohodneme vypnut my)
    }
    @Override
    public void onCreate()
    {
    	
    	Log.e(TAG, "onCreate");
        //na zaciatok si dame do arraylistu znacku o zacati cesty
    	synchronized(lock_db){
        	db = new ArrayList<String>();
        	db.add("RESET;");
        }
        initializeLocationManager();
        (GPSTracker.this).registerReceiver(networkStateReceiver, new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION));
        String androidId = "" + android.provider.Settings.Secure.getString(getContentResolver(), android.provider.Settings.Secure.ANDROID_ID);
        filename="log_"+androidId+".txt";  //nastavime si meno interneho suboru podla ID androidu
        File log_file = new File(getFilesDir(),filename);
    	//ak subor este neexistuje vytvorime si ho
        if (!log_file.exists()){
			try {
				log_file.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
    	}
    }
    
    @Override
    public void onDestroy()
    {
    	//pri vypinani servicu zrusime sledovanie GPS (LocationListener prestane dostavat intend spravy o zmene lokacii)
        Log.e(TAG, "onDestroy");
        super.onDestroy();
        if (mLocationManager != null) {
                try {
                    mLocationManager.removeUpdates(mLocationListener);
                } catch (Exception ex) {
                    Log.i(TAG, "fail to remove location listners, ignore", ex);
                }
        }
        //ulozime si arraylist a subor sa pokusime poslat
        try {
        	synchronized(lock_db){
        		storeData();
        	}
		} catch (IOException e) {
			e.printStackTrace();
		}			
        releaseWakeLock();
    } 
  
    private void initializeLocationManager() {
        Log.e(TAG, "initializeLocationManager");
        if (mLocationManager == null) {
            mLocationManager = (LocationManager) getApplicationContext().getSystemService(Context.LOCATION_SERVICE);
        }
    }
    
    /** metoda, zapise arraylist do suboru a pokusi sa poslat subor cez internet(pri splnenych podmienkach) */
    public void storeData() throws IOException{
		try {
			//uzamkneme subor
			synchronized(lock_file){
			FileOutputStream outputStream;
			outputStream = openFileOutput(filename, Context.MODE_APPEND);			
			for (String point : db)			
				outputStream.write(point.getBytes());
			outputStream.close();
			//po zapisani bodov z arraylistu arraylist vynulujeme
			db.clear();
			}
			
			File log_file = new File(getFilesDir(),filename);
			/*ak sme pripojeny na internet a nas subor prevysuje 3kB 
			alebo bol vypnuty GPS provider(koniec cesty) a nas subor ma aspon 10B (chceme sa vyhnut castemu posielaniu len "RESET" znaciek)
			posleme subor pomocou AsyncTask metody send_file()*/
			if (isNetworkAvailable() && ((log_file.length()>3000) || ((send_because_gps) && (log_file.length()>10)))){ //log_file.lenght() in bytes
					new send_file().execute("");
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    /** metoda, ktora vracia ci sme pripojeny na internet */
    private boolean isNetworkAvailable() {
		ConnectivityManager connectivityManager=(ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
		return activeNetworkInfo != null;
	}
    
    /** metoda, ktora sa zavola ak sme pripojeny na internet a mame dostatok dat v subore. Posielame data zo suboru na server */
    public class send_file extends AsyncTask<String, Void, String> {
    	
    	@Override
    	protected String doInBackground(String... params) {
    		try{
    			//uzamneme subor a data zo suboru prilozime do suboru na serveri s rovnakym nazvom (cize do mojho suboru na serveri)
    		  synchronized (lock_file) {
    			String serverUrl = "**************.sk";
    			String userName = "********";
    			String password ="*****************";
    			
    			JSch jsch = new JSch();
    		
    			Properties config = new Properties();
    			config.put("StrictHostKeyChecking", "no");
    			config.put("compression.s2c", "zlib,none");
    			config.put("compression.c2s", "zlib,none");
    		
    			Session session = jsch.getSession(userName, serverUrl);
    			session.setConfig(config);
    			session.setPort(22);
    			session.setPassword(password);
    			session.connect();
    		
    			ChannelSftp channel = (ChannelSftp) session.openChannel("sftp");
    			channel.connect();
    			
    			File log_file = new File(getFilesDir(),filename);
            	//ak bolo posledne posielanie prerusenie, pokusime sa obnovit posielanie inak cisto prikladame data na novo
    			if (interupted_sending) channel.put(new FileInputStream(log_file), filename,RESUME);
            		else {interupted_sending=true;
            			channel.put(new FileInputStream(log_file), filename,APPEND);
            	}
            	channel.disconnect();
    			session.disconnect();
    			//ak sme sa dostali do tejto casti, pri posielani nedoslo ku vynimke a teda nebolo prerusene
    			interupted_sending=false;
    			//data z interneho suboru mozeme vymazat
    			FileOutputStream outputStream;
	    		outputStream = openFileOutput(filename, Context.MODE_PRIVATE);						
    			outputStream.write(("").getBytes());
    			outputStream.close();
	            send_because_gps=false; //kedze sme data uspesne poslali mozeme nastavit hodnotu send_because_gps na false 
	    		return "Sending Finished";
    		  }
    		} catch(Exception e){
    			System.out.println("Sending interrupted: "+e.toString());
    			return "Sending interrupted: "+e.toString();
    		}
      }      

      @Override
      protected void onPostExecute(String result) {
    	  //po skonceni vypiseme spravu o ukonceni posielania dat
    	  Toast.makeText(GPSTracker.this, "Ended sending GPS data via Network", Toast.LENGTH_SHORT).show();
      }

      @Override
      protected void onPreExecute() { 
    	  //pred zacatim posielania upozornime uzivatela o pripojovani na server a posielani dat
    	  Toast.makeText(GPSTracker.this, "Started sending GPS data via Network", Toast.LENGTH_SHORT).show();
      }

      @Override
      protected void onProgressUpdate(Void... values) {
      }
    }
    
    /**ziskanie wakelocku*/
    private void acquireWakeLock() {
        final PowerManager pm = (PowerManager) this.getSystemService(Context.POWER_SERVICE);
        this.wakeLock = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, TAG);
        this.wakeLock.acquire();
        Log.d( TAG, "Wake lock acquired:" + this.wakeLock);
    }

    /**uvolnenie wakelocku*/
    private void releaseWakeLock() {
        if( this.wakeLock == null )
            return;
        this.wakeLock.release();
        this.wakeLock = null;
        Log.d( TAG, "Wake lock release");
    }
    
}
